/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;

import com.keyware.sonar.cxx.SubscriptionAstVisitor;
import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.AstNodeType;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

/**
 * TODO ReallocMainChecker
 *
 * @author RenFengJiang
 * @date 2024/1/13
 */
@Rule(key = "ReallocMainChecker", name = "使用realloc函数前应先清空敏感信息", description = "使用realloc()函数前先清空该内存块中的敏感信息", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class ReallocMainChecker extends SquidCheck<Grammar> {

    @Override
    public void init() {
        // 指定当前访问器需要访问的节点类型，functionBody（函数）主体节点
        this.subscribeTo(
                CxxGrammarImpl.functionBody
        );
    }

    /**
     * 访问AST节点
     *
     * @param node 要处理的AST节点，该节点类型为通过subscribeTo方法订阅的类型
     */
    @Override
    public void visitNode(@Nonnull AstNode node) {
        IfChildren ifChildren = new IfChildren(this);
        ifChildren.accept(node);

    }

    class IfChildren extends SubscriptionAstVisitor {

        public IfChildren(SquidCheck<Grammar> checker){
            super(checker);
        }

        @Override
        public List<AstNodeType> visitNodeTypes() {
            // 指定当前访问器需要访问的节点类型，这里指定了simpleDeclaration（简单声明）节点类型
            return List.of(CxxGrammarImpl.expression);
        }

        private List<String> lists = new ArrayList();
        /**
         * 检查AST节点
         *
         * @param astNode 要处理的AST节点，该节点类型为通过subscribeTo方法订阅的类型
         */
        @Override
        public void visitNode(AstNode astNode) {
            //判断是否是memset方法调用
            if("memset".equals(astNode.getToken().getValue())){
                //获取到此 方法调用中的参数列表
                List<AstNode> listAsts = astNode.getDescendants(CxxGrammarImpl.expressionList);
                for (AstNode listAst :listAsts) {
                    //将此参数列表存入list中
                    lists.add(listAst.getToken().getValue());
                }
            }
            List<AstNode> astNodeDescendants = astNode.getDescendants(CxxGrammarImpl.postfixExpression);
            for (AstNode ast :astNodeDescendants) {
                //判断是不是realloc方法调用
                if("realloc".equals(ast.getToken().getValue())){
                    //获取到参数列表
                    List<AstNode> astNodes = ast.getDescendants(CxxGrammarImpl.expressionList);
                    for (AstNode as:astNodes) {
                        //获取到参数
                        String name = as.getToken().getValue();
                        //判断参数是否存在在集合中
                        if(!lists.contains(name)){
                            reportIssue(as, "使用realloc函数前应先清楚敏感信息");
                        }
                    }
                }
            }
        }
    }
}
